import os
from pathlib import Path
from string import Template

    # for root, dirs, files in os.walk(str(service_dir)):
    #     print(_(root), [_(d) for d in dirs], [_(f) for f in files])

# def _(original):
#     d = {'name': 'fizzgig'}
#     return Template(original).safe_substitute(d)

#def walk():

IGNORABLES = [
    '.git',
]

class Petrifier():

    def __init__(self, replacements):
        self.replacements = {k: '${' + v + '}'
                             for (k, v) in replacements.items()}

    def __call__(self, original):
        modified = original
        for key, val in self.replacements.items():
            modified = modified.replace(key, val)
        return modified


class Hydrator():

    def __init__(self, replacements):
        self.replacements = replacements

    def __call__(self, original):
        return Template(original).safe_substitute(self.replacements)


def petrify(source_dir, target_dir, replacements, ignorables=None, emptiables=None):

    if ignorables is None:
        ignorables = IGNORABLES

    if emptiables is None:
        emptiables = []

    p = Petrifier(replacements)

    target_path = Path(target_dir)
    if not target_path.exists():
        target_path.mkdir()

    for root, dirs, files in os.walk(str(source_dir)):
        source_root = Path(root)
        target_root = Path(p(root.replace(source_dir, target_dir)))
        print(source_root)
        if not target_root.exists():
            target_root.mkdir()
        for ignorable in ignorables:
            if ignorable in dirs:
                dirs.remove(ignorable)
        for f in files:
            if f in ignorables:
                continue
            source_path = source_root / f
            target_path = target_root / p(f)
            if f in emptiables:
                with target_path.open('w') as t:
                    t.write('')
                continue
            print("  opening ", source_path)
            print("  creating", target_path)
            with source_path.open() as s:
                with target_path.open('w') as t:
                    for line in s:
                        t.write(p(line))

def rehydrate(source_dir, target_dir, replacements):

    h = Hydrator(replacements)

    target_path = Path(target_dir)
    if not target_path.exists():
        target_path.mkdir()

    for root, dirs, files in os.walk(str(source_dir)):
        source_root = Path(root)
        target_root = Path(h(root.replace(source_dir, target_dir)))
        print(source_root)
        if not target_root.exists():
            target_root.mkdir()
        for f in files:
            source_path = source_root / f
            target_path = target_root / h(f)
            print("  opening ", source_path)
            print("  creating", target_path)
            with source_path.open() as s:
                with target_path.open('w') as t:
                    for line in s:
                        t.write(h(line))



#!/usr/bin/env python3

from core import petrify, rehydrate, IGNORABLES
from shutil import rmtree
from pathlib import Path

def main():

    target = Path('service_skeleton')
    if target.exists():
        rmtree(str(target))

    replacements = {
        'prism': 'name',
        'Prism': 'Name',
        'Jeff Klukas': 'author',
        'jklukas@simple.com': 'email',
    }

    ignorables = IGNORABLES + [
        'target',
        '.ensime_cache',
    ]

    emptiables = [
        'banner.txt'
    ]

    petrify('prism_original', 'service_skeleton', replacements, ignorables)

    replacements = {
        'name': 'spoo',
        'Name': 'Spoo',
        'author': 'Keldor Skeletor',
        'email': 'keldor@skeletor.net',
    }

    rehydrate('service_skeleton', 'spoo', replacements)

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# # Get the long description from the relevant file
# with open(path.join(here, 'DESCRIPTION.rst'), encoding='utf-8') as f:
#     long_description = f.read()

setup(
    name='skelt',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version='0.0.1',

    description='Creating and rehydrating code skeletons since 2014',
    # long_description=long_description,

    # The project's main homepage.
    url='https://github.com/jklukas/skelt',

    # Author details
    author='Jeff Klukas',
    author_email='jeff@klukas.net',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Code Generators',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],

    # What does your project relate to?
    keywords='library setup generator skeleton',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=['skelt'],

)
